'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _components = require('../components');

var _utils = require('../utils');

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Darkroom Kitchen Sink
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * A bit overkill but use-able
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */

var canvasWidth = 300;
var canvasHeight = 300;

function fileController() {
  var thread = arguments.length <= 0 || arguments[0] === undefined ? { source: null } : arguments[0];
  var action = arguments[1];

  switch (action.type) {
    case 'SET_FILE':
      return Object.assign({}, thread, {
        source: action.file,
        angle: 0
      });
    default:
      return thread;
  }
}
function imageController() {
  var thread = arguments.length <= 0 || arguments[0] === undefined ? { crop: false, source: null, angle: 0 } : arguments[0];
  var action = arguments[1];

  switch (action.type) {
    case 'ROTATE_LEFT':
      return Object.assign({}, thread, {
        angle: thread.angle - 90
      });
    case 'ROTATE_RIGHT':
      return Object.assign({}, thread, {
        angle: thread.angle + 90
      });
    case 'START_CROPPING':
      return Object.assign({}, thread, {
        crop: true
      });
    case 'STOP_CROPPING':
      return Object.assign({}, thread, {
        crop: false
      });
    case 'CONFIRM_CROP':
      return Object.assign({}, thread, {
        crop: false,
        source: action.image
      });
    default:
      return thread;
  }
}
function readFile(file, done) {
  var reader = new FileReader();
  reader.onload = function (e) {
    return done(e.target.result);
  };
  reader.readAsDataURL(file);
}

/**
 * Darkroom API methods
 *
 * Set image/width/height
 * Set Rotation
 * Use React.Children.foreach can iterate through each item
 * for a declarative API
 * Image rotation -> Should that be state?  Or should we set that
 * independantly and allow for image rotation to be easily set?
 *
 * Controls to be a separate and optional component for maximum
 * re-usability.
 */

var KitchenSink = function (_React$Component) {
  _inherits(KitchenSink, _React$Component);

  function KitchenSink(props) {
    _classCallCheck(this, KitchenSink);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(KitchenSink).call(this, props));

    _this.state = { step: 0, thread: [{ crop: false, source: null, angle: 0 }] };
    ['onFileChange', 'update', 'onRedo', 'onUndo', 'onRotateLeft', 'onCropStart', 'onCropConfirm', 'onCropCancel', 'onRotateRight'].forEach(function (method) {
      _this[method] = _this[method].bind(_this);
    });
    return _this;
  }

  _createClass(KitchenSink, [{
    key: 'componentDidUpdate',
    value: function componentDidUpdate() {
      console.timeEnd('State changed');
    }
  }, {
    key: 'update',
    value: function update(action) {
      var state = this.state;
      var nextThread = void 0;
      var nextStep = state.thread.length;
      var newState = void 0;
      var newThread = void 0;

      switch (action.type) {
        case "SET_FILE":
          nextThread = fileController(state.thread[state.step], action);
          break;

        case "UNDO":
          nextStep = state.step - 1;
          break;

        case "REDO":
          nextStep = state.step + 1;
          break;

        case "ROTATE_LEFT":
        case "ROTATE_RIGHT":
        case "START_CROPPING":
        case "STOP_CROPPING":
        case "CONFIRM_CROP":
          nextThread = imageController(state.thread[state.step], action);
          break;

      }

      if (action.type !== "UNDO" && action.type !== "REDO" && state.step > 0 && state.step < state.thread.length - 1) {
        newThread = [].concat(_toConsumableArray(state.thread.slice(0, state.step)), [nextThread]);
        nextStep = newThread.length - 1;
      } else {
        newThread = nextThread ? [].concat(_toConsumableArray(state.thread), [nextThread]) : [].concat(state.thread);
      }

      newState = Object.assign({}, state, {
        step: nextStep,
        thread: newThread
      });
      console.time('State changed');
      this.setState(newState);
    }
  }, {
    key: 'onFileChange',
    value: function onFileChange(e) {
      var _this2 = this;

      readFile(e.target.files[0], function (file) {
        _this2.update({ type: 'SET_FILE', file: file });
      });
    }
  }, {
    key: 'onUndo',
    value: function onUndo() {
      this.update({ type: 'UNDO' });
    }
  }, {
    key: 'onRedo',
    value: function onRedo() {
      this.update({ type: 'REDO' });
    }
  }, {
    key: 'onRotateLeft',
    value: function onRotateLeft() {
      this.update({ type: 'ROTATE_LEFT' });
    }
  }, {
    key: 'onRotateRight',
    value: function onRotateRight() {
      this.update({ type: 'ROTATE_RIGHT' });
    }
  }, {
    key: 'onCropStart',
    value: function onCropStart() {
      this.update({ type: 'START_CROPPING' });
    }
  }, {
    key: 'onCropCancel',
    value: function onCropCancel() {
      this.update({ type: 'STOP_CROPPING' });
    }
  }, {
    key: 'onCropConfirm',
    value: function onCropConfirm() {
      var _this3 = this;

      var source = this.state.thread[this.state.step].source;
      var _refs$canvasWrapper$c = this.refs.canvasWrapper.cropBox;
      var x = _refs$canvasWrapper$c.x;
      var y = _refs$canvasWrapper$c.y;
      var width = _refs$canvasWrapper$c.width;
      var height = _refs$canvasWrapper$c.height;


      var newImage = _utils.Transform.cropImage(source, { x: x, y: y, width: width, height: height }, { width: canvasWidth, height: canvasHeight }).then(function (image) {
        return _this3.update({ type: 'CONFIRM_CROP', image: image });
      });
    }
  }, {
    key: 'render',
    value: function render() {
      var _this4 = this;

      var current = this.state.thread[this.state.step];
      var angle = current.angle;
      var source = current.source;
      var crop = current.crop;

      var hasFile = source !== null;

      var selectFile = function selectFile() {
        _this4.refs.fileselect.click();
      };

      return _react2.default.createElement(
        'div',
        { style: { padding: "0 1em" } },
        _react2.default.createElement(
          'header',
          null,
          _react2.default.createElement(
            'h2',
            null,
            'React Darkroom Kitchen Sink v0.3.5'
          ),
          _react2.default.createElement(
            'a',
            { href: 'https://github.com/MattMcFarland/react-darkroom' },
            'https://github.com/MattMcFarland/react-darkroom'
          ),
          _react2.default.createElement('hr', null)
        ),
        _react2.default.createElement(
          'div',
          { style: { padding: "1em" } },
          _react2.default.createElement(
            _components.Darkroom,
            null,
            _react2.default.createElement(
              _components.Toolbar,
              null,
              _react2.default.createElement(
                'button',
                { onClick: selectFile, 'data-tipsy': 'Select Image', className: 'tipsy tipsy--s' },
                _react2.default.createElement('span', { className: 'icon icon-image' }),
                _react2.default.createElement('input', { type: 'file', ref: 'fileselect', onChange: this.onFileChange, style: { display: 'none' } })
              ),
              _react2.default.createElement(
                _components.History,
                { step: this.state.step, length: this.state.thread.length - 1 },
                _react2.default.createElement(
                  'button',
                  {
                    action: 'back',
                    onClick: this.onUndo,
                    'data-ifEmpty': 'disable',
                    'data-tipsy': 'Undo',
                    className: 'tipsy tipsy--sw' },
                  _react2.default.createElement('span', { className: 'icon icon-undo2' })
                ),
                _react2.default.createElement(
                  'button',
                  {
                    action: 'forward',
                    onClick: this.onRedo,
                    'data-ifEmpty': 'disable',
                    'data-tipsy': 'Redo',
                    className: 'tipsy tipsy--sw' },
                  _react2.default.createElement('span', { className: 'icon icon-redo2' })
                )
              ),
              _react2.default.createElement(
                'button',
                {
                  disabled: !hasFile,
                  onClick: this.onRotateLeft,
                  'data-tipsy': 'Rotate Left',
                  className: 'tipsy tipsy--sw' },
                _react2.default.createElement('span', { className: 'icon icon-undo' })
              ),
              _react2.default.createElement(
                'button',
                {
                  disabled: !hasFile,
                  onClick: this.onRotateRight,
                  'data-tipsy': 'Rotate Right',
                  className: 'tipsy tipsy--sw' },
                _react2.default.createElement('span', { className: 'icon icon-redo' })
              ),
              _react2.default.createElement(
                _components.CropMenu,
                { isCropping: crop },
                _react2.default.createElement(
                  'button',
                  {
                    disabled: !hasFile,
                    'data-showOnlyWhen': 'croppingIsOff',
                    onClick: this.onCropStart,
                    'data-tipsy': 'Crop',
                    className: 'tipsy tipsy--sw' },
                  _react2.default.createElement('span', { className: 'icon icon-crop' })
                ),
                _react2.default.createElement(
                  'button',
                  { disabled: !hasFile, showOnlyWhen: 'croppingIsOn', style: { color: 'cyan' } },
                  _react2.default.createElement('span', { className: 'icon icon-crop' })
                ),
                _react2.default.createElement(
                  'button',
                  {
                    disabled: !hasFile,
                    'data-showOnlyWhen': 'croppingIsOn',
                    onClick: this.onCropConfirm,
                    style: { color: 'green' },
                    'data-tipsy': 'Confirm',
                    className: 'tipsy tipsy--sw' },
                  _react2.default.createElement('span', { className: 'icon icon-checkmark' })
                ),
                _react2.default.createElement(
                  'button',
                  {
                    disabled: !hasFile,
                    'data-showOnlyWhen': 'croppingIsOn',
                    onClick: this.onCropCancel,
                    style: { color: 'red' },
                    'data-tipsy': 'Cancel',
                    className: 'tipsy tipsy--sw' },
                  _react2.default.createElement('span', { className: 'icon icon-cross' })
                )
              ),
              _react2.default.createElement(
                'button',
                { disabled: !hasFile, onClick: this.onSave, 'data-tipsy': 'Save', className: 'tipsy tipsy--sw' },
                _react2.default.createElement('span', { className: 'icon icon-floppy-disk' })
              )
            ),
            _react2.default.createElement(
              _components.Canvas,
              {
                ref: 'canvasWrapper',
                crop: crop,
                source: source,
                angle: angle,
                width: canvasWidth,
                height: canvasHeight },
              _react2.default.createElement(_components.FilePicker, { hasFile: hasFile, onChange: this.onFileChange })
            )
          )
        )
      );
    }
  }]);

  return KitchenSink;
}(_react2.default.Component);

exports.default = KitchenSink;