'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Transform = exports.Transform = function () {
  function Transform() {
    _classCallCheck(this, Transform);
  }

  _createClass(Transform, null, [{
    key: 'constrainProportions',
    value: function constrainProportions(from, to) {

      var minRatio = Math.min(to.height / from.height, to.width / from.width);
      return {
        width: from.width > to.width ? from.width * minRatio : from.width,
        height: from.height > to.height ? from.height * minRatio : from.height
      };
    }
  }, {
    key: 'centerRect',
    value: function centerRect(rect, container) {
      return {
        x: container.width * 0.5 - rect.width * 0.5,
        y: container.height * 0.5 - rect.height * 0.5
      };
    }
  }, {
    key: 'clearCanvas',
    value: function clearCanvas(canvas, ctx) {
      ctx.clearRect(0, 0, canvas.width, canvas.height);
      ctx.save();
    }
  }, {
    key: 'rotateImage',
    value: function rotateImage(ctx, angle) {
      var canvas = ctx.canvas;
      ctx.translate(canvas.width * 0.5, canvas.height * 0.5);
      ctx.rotate(angle * Math.PI / 180);
      ctx.translate(-canvas.width * 0.5, -canvas.height * 0.5);
    }
  }, {
    key: 'renderImage',
    value: function renderImage(ctx, img, position, boundRect) {

      ctx.drawImage(img, position.x, position.y, boundRect.width, boundRect.height);
      ctx.restore();
    }
  }, {
    key: 'loadBitmapData',
    value: function loadBitmapData(source) {
      return new Promise(function (resolve, reject) {
        try {
          (function () {
            var img = new Image();
            img.onload = function () {
              resolve(img);
            };
            img.src = Object.assign(source);
          })();
        } catch (er) {
          reject(er);
        }
      });
    }
  }, {
    key: 'renderCentered',
    value: function renderCentered(ctx, image, imgRect, boundRect) {
      var scaledRect = Transform.constrainProportions(imgRect, boundRect);
      var position = Transform.centerRect(scaledRect, boundRect);

      Transform.renderImage(ctx, image, position, scaledRect);
    }
  }, {
    key: 'getRealDimensions',
    value: function getRealDimensions(cropRect, imgRect) {
      return {
        x: (cropRect.x - imgRect.x) / imgRect.width,
        y: (cropRect.y - imgRect.y) / imgRect.height,
        width: cropRect.width / imgRect.width,
        height: cropRect.height / imgRect.height
      };
    }

    /**
     * Crop an image and return the new image object
     * @param image {Image}
     * @param cropRect  {Object}
     * @param     cropRect.x {number}
     * @param     cropRect.y {number}
     * @param     cropRect.width {number}
     * @param     cropRect.height {number}
     * @param boundRect {Object}
     * @param     boundRect.width {number}
     * @param     boundRect.height {number}
     * @param angle {number}
     * @returns {Promise}
     */

  }, {
    key: 'cropImage',
    value: function cropImage(image, cropRect, boundRect, angle) {
      console.log('crop image', cropRect, boundRect);
      return new Promise(function (resolve, reject) {
        var scaledCanvas = document.createElement('canvas');
        scaledCanvas.width = boundRect.width;
        scaledCanvas.height = boundRect.height;
        var sctx = scaledCanvas.getContext('2d');

        Transform.loadBitmapData(image).then(function (img) {
          Transform.rotateImage(sctx, angle);
          Transform.renderCentered(sctx, img, img, boundRect);
          var scaledImage = new Image();
          scaledImage.onload = function () {

            var croppedCanvas = document.createElement('canvas');
            var ctx = croppedCanvas.getContext('2d');

            croppedCanvas.width = boundRect.width;
            croppedCanvas.height = boundRect.height;

            ctx.drawImage(scaledImage, cropRect.x, cropRect.y, cropRect.width, cropRect.height, 0, 0, boundRect.width, boundRect.height);

            resolve(croppedCanvas.toDataURL("image/png"));
          };
          scaledImage.src = scaledCanvas.toDataURL("image/png");
        });
      });
    }
  }]);

  return Transform;
}();